from typing import Any, Optional

from pydantic import BaseModel, ConfigDict


### Plane Webhook Models
class Credentials(BaseModel):
    client_id: str
    client_secret: str
    app_installation_id: str
    base_url: Optional[str] = "https://api.plane.so"
    redirect_uri: str


class Actor(BaseModel):
    id: str
    first_name: str
    last_name: str
    email: str
    avatar: str
    avatar_url: Optional[str]
    display_name: str


class Activity(BaseModel):
    field: Optional[str] = None
    new_value: Optional[Any]
    old_value: Optional[Any]
    actor: Actor
    old_identifier: Optional[Any]
    new_identifier: Optional[Any]


class OAuthResponse(BaseModel):
    access_token: str
    expires_in: int
    token_type: str
    scope: str
    app_installation_id: str
    workspace_id: str
    bot_user_id: str
    workspace_slug: str


class EventData(BaseModel):
    """Base model for webhook event data with common fields"""

    model_config = ConfigDict(extra="allow")  # Allow additional fields

    id: str
    project: Optional[str] = None  # Some events might not have project
    workspace: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None


class IssueEventData(EventData):
    """Specific model for issue events"""

    name: Optional[str] = None
    description_html: Optional[str] = None
    priority: Optional[str] = None
    assignees: Optional[list] = None
    labels: Optional[list] = None
    state: Optional[dict] = None


class CommentEventData(EventData):
    """Specific model for comment events"""

    issue: Optional[str] = None  # For comment events, this refers to the issue ID
    comment_html: Optional[str] = None
    comment_stripped: Optional[str] = None


class WebhookEvent(BaseModel):
    event: str
    action: str
    webhook_id: str
    workspace_id: str
    data: Any
    activity: Activity


### Agent Models
class Context(BaseModel):
    entity_name: str
    entity_id: str
    issue_id: str
    event: WebhookEvent
    workspace_slug: str
    workspace_id: str
    project_id: Optional[str] = None
    issue_payload: Optional[dict] = None


class TokenDetails(BaseModel):
    access_token: str
    expires_in: int
    token_type: str
    scope: str


class WorkspaceDetails(BaseModel):
    slug: str
    id: str
    bot_user_id: str



from .agent import PlaneAgent
from .handlers import AgentEventHandler, IssueEventHandler
from .models import Context, Credentials

__all__ = [
    "PlaneAgent",
    "Credentials",
    "Context",
    "AgentEventHandler",
    "IssueEventHandler",
]



from abc import ABC, abstractmethod
from typing import Callable

from plane.api.work_item_comments_api import WorkItemCommentsApi
from plane.api.work_items_api import WorkItemsApi
from plane.api_client import ApiClient
from plane.models.issue_comment_create_request import IssueCommentCreateRequest

from plane_adk.agent.models import (
    Context,
    WebhookEvent,
    WorkspaceDetails,
    IssueEventData,
    CommentEventData,
)


class AgentEventHandler(ABC):
    supported_events: list[str] = []
    callbacks: dict[str, Callable] = {}

    def __init__(self, plane_client: ApiClient, workspace_details: WorkspaceDetails):
        self.plane_client = plane_client
        self.workspace_details = workspace_details

    def on(self, event_name: str, callback: Callable):
        if event_name not in self.supported_events:
            raise ValueError(f"Event {event_name} is not supported")
        self.callbacks[event_name] = callback

    @abstractmethod
    def process_webhook(self, event: dict):
        pass

    @abstractmethod
    def progress(self, message: str):
        pass

    def get_client(self) -> ApiClient:
        return self.plane_client


class IssueEventHandler(AgentEventHandler):
    supported_events: list[str] = ["assigned", "mentioned", "unassigned"]
    context: Context = None

    def process_webhook(self, event: dict):
        event_model = WebhookEvent(**event)
        if event_model.event == "issue":
            self._process_issue(event_model)
        elif event_model.event == "issue_comment":
            self._process_issue_comment(event_model)

    def _process_issue(self, event_model: WebhookEvent):
        event_data = IssueEventData(**event_model.data)
        self.context = Context(
            issue_id=event_data.id,
            event=event_model,
            workspace_id=event_model.workspace_id,
            workspace_slug=self.workspace_details.slug,
            project_id=event_data.project,
            entity_name="issue",
            entity_id=event_data.id,
            issue_payload=WorkItemsApi(self.plane_client).retrieve_work_item(
                event_data.id,
                event_data.project,
                self.workspace_details.slug,
            ),
        )

        if event_model.activity.field == "assignees":
            if (
                event_model.activity.new_identifier
                == self.workspace_details.bot_user_id
            ):
                if self.callbacks["assigned"]:
                    self.callbacks["assigned"](self, self.context)

    def _process_issue_comment(self, event_model: WebhookEvent):
        event_data = CommentEventData(**event_model.data)
        self.context = Context(
            issue_id=event_data.issue,
            event=event_model,
            workspace_id=event_model.workspace_id,
            workspace_slug=self.workspace_details.slug,
            project_id=event_data.project,
            entity_name="comment",
            entity_id=event_data.id,
            issue_payload=WorkItemsApi(self.plane_client).retrieve_work_item(
                event_data.issue,
                event_data.project,
                self.workspace_details.slug,
            ),
        )
        bot_mention = f'"{self.workspace_details.bot_user_id}"'

        if not event_model.activity:
            return

        if event_model.activity.field == "description" and (
            bot_mention in event_model.activity.new_value
            and (
                event_model.activity.old_value is None
                or bot_mention not in event_model.activity.old_value
            )
        ):
            if self.callbacks["mentioned"]:
                self.callbacks["mentioned"](self, self.context)

    def progress(self, message: str):
        WorkItemCommentsApi(self.plane_client).create_work_item_comment(
            self.context.issue_id,
            self.context.project_id,
            self.context.workspace_slug,
            IssueCommentCreateRequest(comment_html=f"<p>{message}</p>"),
        )



from plane.api_client import ApiClient
from plane.configuration import Configuration
from plane.oauth.api import OAuthApi

from plane_adk.agent.handlers import AgentEventHandler, IssueEventHandler
from plane_adk.agent.models import Credentials, TokenDetails, WorkspaceDetails


class PlaneAgent:
    issue: AgentEventHandler
    token_details: TokenDetails
    workspace_details: WorkspaceDetails

    def __init__(self, credentials: Credentials):
        self.credentials = credentials
        self._init_token()
        configuration = Configuration(
            host=credentials.base_url,
            access_token=self.token_details.access_token,
        )
        api_client = ApiClient(configuration)
        self.issue = IssueEventHandler(api_client, self.workspace_details)

    def process_webhook(self, webhook: dict):
        if webhook["event"] in ["issue", "issue_comment"]:
            self.issue.process_webhook(webhook)

    def _init_token(self):
        oauth_api = OAuthApi(
            client_id=self.credentials.client_id,
            client_secret=self.credentials.client_secret,
            redirect_uri=self.credentials.redirect_uri,
            base_url=self.credentials.base_url,
        )
        token_response = oauth_api.get_bot_token(self.credentials.app_installation_id)
        self.token_details = TokenDetails(
            access_token=token_response.access_token,
            expires_in=token_response.expires_in,
            token_type=token_response.token_type,
            scope=token_response.scope,
        )
        app_installation = oauth_api.get_app_installation(
            self.token_details.access_token, self.credentials.app_installation_id
        )
        self.workspace_details = WorkspaceDetails(
            slug=app_installation.workspace_detail.slug,
            id=app_installation.workspace,
            bot_user_id=app_installation.app_bot,
        )
